@@ -13,8 +13,9 @@
 #include <utility>
 #include <vector>
 #include <algorithm>
+#include <memory>
 
-#include "xarray_base.hpp"
+#include "xcontainer.hpp"
 #include "xsemantic.hpp"
 
 namespace xt
@@ -24,14 +25,16 @@ namespace xt
      * xarray declaration *
      **********************/
 
-    template <class T>
+    template <class T, class EA = std::allocator<T>, class SA = std::allocator<typename std::vector<T, EA>::size_type>>
     class xarray;
 
-    template <class T>
-    struct array_inner_types<xarray<T>>
+    template <class T, class EA, class SA>
+    struct xcontainer_inner_types<xarray<T, EA, SA>>
     {
-        using container_type = std::vector<T>;
-        using temporary_type = xarray<T>;
+        using container_type = std::vector<T, EA>;
+        using shape_type = std::vector<typename container_type::size_type, SA>;
+        using strides_type = shape_type;
+        using temporary_type = xarray<T, EA, SA>;
     };
 
     /**
@@ -44,16 +47,16 @@ namespace xt
      *
      * @tparam T The type of objects stored in the container.
      */
-    template <class T>
-    class xarray : public xarray_base<xarray<T>>,
-                   public xarray_semantic<xarray<T>>
+    template <class T, class EA, class SA>
+    class xarray : public xcontainer<xarray<T, EA, SA>>,
+                   public xcontainer_semantic<xarray<T, EA, SA>>
     {
 
     public:
 
-        using self_type = xarray<T>;
-        using base_type = xarray_base<self_type>;
-        using semantic_base = xarray_semantic<self_type>;
+        using self_type = xarray<T, EA, SA>;
+        using base_type = xcontainer<self_type>;
+        using semantic_base = xcontainer_semantic<self_type>;
         using container_type = typename base_type::container_type;
         using value_type = typename base_type::value_type;
         using reference = typename base_type::reference;
@@ -63,15 +66,13 @@ namespace xt
         using shape_type = typename base_type::shape_type;
         using strides_type = typename base_type::strides_type;
 
-        using closure_type = const self_type&;
-
         xarray();
         explicit xarray(const shape_type& shape, layout l = layout::row_major);
         explicit xarray(const shape_type& shape, const_reference value, layout l = layout::row_major);
         explicit xarray(const shape_type& shape, const strides_type& strides);
         explicit xarray(const shape_type& shape, const strides_type& strides, const_reference value);
 
-        explicit xarray(const T& t);
+        xarray(const value_type& t);
         xarray(std::initializer_list<T> t);
         xarray(std::initializer_list<std::initializer_list<T>> t);
         xarray(std::initializer_list<std::initializer_list<std::initializer_list<T>>> t);
@@ -96,24 +97,26 @@ namespace xt
 
         container_type m_data;
 
-        container_type& data_impl();
-        const container_type& data_impl() const;
+        container_type& data_impl() noexcept;
+        const container_type& data_impl() const noexcept;
 
-        friend class xarray_base<xarray<T>>;
+        friend class xcontainer<xarray<T, EA, SA>>;
     };
 
     /******************************
      * xarray_adaptor declaration *
      ******************************/
 
-    template <class C>
+    template <class C, class EA = std::allocator<typename C::value_type>, class SA = std::allocator<typename C::size_type>>
     class xarray_adaptor;
 
-    template <class C>
-    struct array_inner_types<xarray_adaptor<C>>
+    template <class C, class EA, class SA>
+    struct xcontainer_inner_types<xarray_adaptor<C, EA, SA>>
     {
         using container_type = C;
-        using temporary_type = xarray<typename C::value_type>;
+        using shape_type = std::vector<typename container_type::size_type, SA>;
+        using strides_type = shape_type;
+        using temporary_type = xarray<typename C::value_type, EA, SA>;
     };
 
     /**
@@ -128,22 +131,20 @@ namespace xt
      *
      * @tparam C The container type to adapt.
      */
-    template <class C>
-    class xarray_adaptor : public xarray_base<xarray_adaptor<C>>,
-                           public xadaptor_semantic<xarray_adaptor<C>>
+    template <class C, class EA, class SA>
+    class xarray_adaptor : public xcontainer<xarray_adaptor<C, EA, SA>>,
+                           public xadaptor_semantic<xarray_adaptor<C, EA, SA>>
     {
 
     public:
 
-        using self_type = xarray_adaptor<C>;
-        using base_type = xarray_base<self_type>;
+        using self_type = xarray_adaptor<C, EA, SA>;
+        using base_type = xcontainer<self_type>;
         using semantic_base = xadaptor_semantic<self_type>;
         using container_type = typename base_type::container_type;
         using shape_type = typename base_type::shape_type;
         using strides_type = typename base_type::strides_type;
 
-        using closure_type = const self_type&;
-
         xarray_adaptor(container_type& data);
         xarray_adaptor(container_type& data, const shape_type& shape, layout l = layout::row_major);
         xarray_adaptor(container_type& data, const shape_type& shape, const strides_type& strides);
@@ -163,14 +164,14 @@ namespace xt
 
         container_type& m_data;
 
-        container_type& data_impl();
-        const container_type& data_impl() const;
+        container_type& data_impl() noexcept;
+        const container_type& data_impl() const noexcept;
 
-        using temporary_type = typename array_inner_types<self_type>::temporary_type;
+        using temporary_type = typename xcontainer_inner_types<self_type>::temporary_type;
         void assign_temporary_impl(temporary_type& tmp);
 
-        friend class xarray_base<xarray_adaptor<C>>;
-        friend class xadaptor_semantic<xarray_adaptor<C>>;
+        friend class xcontainer<xarray_adaptor<C, EA, SA>>;
+        friend class xadaptor_semantic<xarray_adaptor<C, EA, SA>>;
     };
 
     /*************************
@@ -184,8 +185,8 @@ namespace xt
     /**
      * Allocates an uninitialized xarray that holds 0 element.
      */
-    template <class T>
-    inline xarray<T>::xarray()
+    template <class T, class EA, class SA>
+    inline xarray<T, EA, SA>::xarray()
         : base_type(), m_data(1, value_type())
     {
     }
@@ -196,8 +197,8 @@ namespace xt
      * @param shape the shape of the xarray
      * @param l the layout of the xarray
      */
-    template <class T>
-    inline xarray<T>::xarray(const shape_type& shape, layout l)
+    template <class T, class EA, class SA>
+    inline xarray<T, EA, SA>::xarray(const shape_type& shape, layout l)
         : base_type()
     {
         base_type::reshape(shape, l);
@@ -210,8 +211,8 @@ namespace xt
      * @param value the value of the elements
      * @param l the layout of the xarray
      */
-    template <class T>
-    inline xarray<T>::xarray(const shape_type& shape, const_reference value, layout l)
+    template <class T, class EA, class SA>
+    inline xarray<T, EA, SA>::xarray(const shape_type& shape, const_reference value, layout l)
         : base_type()
     {
         base_type::reshape(shape, l);
@@ -223,8 +224,8 @@ namespace xt
      * @param shape the shape of the xarray
      * @param strides the strides of the xarray
      */
-    template <class T>
-    inline xarray<T>::xarray(const shape_type& shape, const strides_type& strides)
+    template <class T, class EA, class SA>
+    inline xarray<T, EA, SA>::xarray(const shape_type& shape, const strides_type& strides)
         : base_type()
     {
         base_type::reshape(shape, strides);
@@ -237,8 +238,8 @@ namespace xt
      * @param strides the strides of the xarray
      * @param value the value of the elements
      */
-    template <class T>
-    inline xarray<T>::xarray(const shape_type& shape, const strides_type& strides, const_reference value)
+    template <class T, class EA, class SA>
+    inline xarray<T, EA, SA>::xarray(const shape_type& shape, const strides_type& strides, const_reference value)
         : base_type()
     {
         base_type::reshape(shape, strides);
@@ -250,8 +251,8 @@ namespace xt
      * specified value.
      * @param t the value of the element
      */
-    template <class T>
-    inline xarray<T>::xarray(const T& t)
+    template <class T, class EA, class SA>
+    inline xarray<T, EA, SA>::xarray(const value_type& t)
         : base_type()
     {
         base_type::reshape(xt::shape<shape_type>(t), layout::row_major);
@@ -267,8 +268,8 @@ namespace xt
      * Allocates a one-dimensional xarray.
      * @param t the elements of the xarray
      */
-    template <class T>
-    inline xarray<T>::xarray(std::initializer_list<T> t)
+    template <class T, class EA, class SA>
+    inline xarray<T, EA, SA>::xarray(std::initializer_list<T> t)
         : base_type()
     {
         base_type::reshape(xt::shape<shape_type>(t), layout::row_major);
@@ -279,8 +280,8 @@ namespace xt
      * Allocates a two-dimensional xarray.
      * @param t the elements of the xarray
      */
-    template <class T>
-    inline xarray<T>::xarray(std::initializer_list<std::initializer_list<T>> t)
+    template <class T, class EA, class SA>
+    inline xarray<T, EA, SA>::xarray(std::initializer_list<std::initializer_list<T>> t)
         : base_type()
     {
         base_type::reshape(xt::shape<shape_type>(t), layout::row_major);
@@ -291,8 +292,8 @@ namespace xt
      * Allocates a three-dimensional xarray.
      * @param t the elements of the xarray
      */
-    template <class T>
-    inline xarray<T>::xarray(std::initializer_list<std::initializer_list<std::initializer_list<T>>> t)
+    template <class T, class EA, class SA>
+    inline xarray<T, EA, SA>::xarray(std::initializer_list<std::initializer_list<std::initializer_list<T>>> t)
         : base_type()
     {
         base_type::reshape(xt::shape<shape_type>(t), layout::row_major);
@@ -303,8 +304,8 @@ namespace xt
      * Allocates a four-dimensional xarray.
      * @param t the elements of the xarray
      */
-    template <class T>
-    inline xarray<T>::xarray(std::initializer_list<std::initializer_list<std::initializer_list<std::initializer_list<T>>>> t)
+    template <class T, class EA, class SA>
+    inline xarray<T, EA, SA>::xarray(std::initializer_list<std::initializer_list<std::initializer_list<std::initializer_list<T>>>> t)
         : base_type()
     {
         base_type::reshape(xt::shape<shape_type>(t), layout::row_major);
@@ -315,8 +316,8 @@ namespace xt
      * Allocates a five-dimensional xarray.
      * @param t the elements of the xarray
      */
-    template <class T>
-    inline xarray<T>::xarray(std::initializer_list<std::initializer_list<std::initializer_list<std::initializer_list<std::initializer_list<T>>>>> t)
+    template <class T, class EA, class SA>
+    inline xarray<T, EA, SA>::xarray(std::initializer_list<std::initializer_list<std::initializer_list<std::initializer_list<std::initializer_list<T>>>>> t)
         : base_type()
     {
         base_type::reshape(xt::shape<shape_type>(t), layout::row_major);
@@ -324,6 +325,7 @@ namespace xt
     }
     //@}
 
+
     /**
      * @name Extended copy semantic
      */
@@ -331,9 +333,9 @@ namespace xt
     /**
      * The extended copy constructor.
      */
-    template <class T>
+    template <class T, class EA, class SA>
     template <class E>
-    inline xarray<T>::xarray(const xexpression<E>& e)
+    inline xarray<T, EA, SA>::xarray(const xexpression<E>& e)
         : base_type()
     {
         semantic_base::assign(e);
@@ -342,22 +344,22 @@ namespace xt
     /**
      * The extended assignment operator.
      */
-    template <class T>
+    template <class T, class EA, class SA>
     template <class E>
-    inline auto xarray<T>::operator=(const xexpression<E>& e) -> self_type&
+    inline auto xarray<T, EA, SA>::operator=(const xexpression<E>& e) -> self_type&
     {
         return semantic_base::operator=(e);
     }
     //@}
 
-    template <class T>
-    inline auto xarray<T>::data_impl() -> container_type&
+    template <class T, class EA, class SA>
+    inline auto xarray<T, EA, SA>::data_impl() noexcept -> container_type&
     {
         return m_data;
     }
 
-    template <class T>
-    inline auto xarray<T>::data_impl() const -> const container_type&
+    template <class T, class EA, class SA>
+    inline auto xarray<T, EA, SA>::data_impl() const noexcept -> const container_type&
     {
         return m_data;
     }
@@ -374,8 +376,8 @@ namespace xt
      * Constructs an xarray_adaptor of the given stl-like container.
      * @param data the container to adapt
      */
-    template <class C>
-    inline xarray_adaptor<C>::xarray_adaptor(container_type& data)
+    template <class C, class EA, class SA>
+    inline xarray_adaptor<C, EA, SA>::xarray_adaptor(container_type& data)
         : base_type(), m_data(data)
     {
     }
@@ -387,8 +389,8 @@ namespace xt
      * @param shape the shape of the xarray_adaptor
      * @param l the layout of the xarray_adaptor
      */
-    template <class C>
-    inline xarray_adaptor<C>::xarray_adaptor(container_type& data, const shape_type& shape, layout l)
+    template <class C, class EA, class SA>
+    inline xarray_adaptor<C, EA, SA>::xarray_adaptor(container_type& data, const shape_type& shape, layout l)
         : base_type(), m_data(data)
     {
         base_type::reshape(shape, l);
@@ -401,24 +403,24 @@ namespace xt
      * @param shape the shape of the xarray_adaptor
      * @param strides the strides of the xarray_adaptor
      */
-    template <class C>
-    inline xarray_adaptor<C>::xarray_adaptor(container_type& data, const shape_type& shape, const strides_type& strides)
+    template <class C, class EA, class SA>
+    inline xarray_adaptor<C, EA, SA>::xarray_adaptor(container_type& data, const shape_type& shape, const strides_type& strides)
         : base_type(), m_data(data)
     {
         base_type::reshape(shape, strides);
     }
     //@}
 
-    template <class C>
-    inline xarray_adaptor<C>& xarray_adaptor<C>::operator=(const xarray_adaptor& rhs)
+    template <class C, class EA, class SA>
+    inline xarray_adaptor<C, EA, SA>& xarray_adaptor<C, EA, SA>::operator=(const xarray_adaptor& rhs)
     {
         base_type::operator=(rhs);
         m_data = rhs.m_data;
         return *this;
     }
 
-    template <class C>
-    inline xarray_adaptor<C>& xarray_adaptor<C>::operator=(xarray_adaptor&& rhs)
+    template <class C, class EA, class SA>
+    inline xarray_adaptor<C, EA, SA>& xarray_adaptor<C, EA, SA>::operator=(xarray_adaptor&& rhs)
     {
         base_type::operator=(std::move(rhs));
         m_data = rhs.m_data;
@@ -432,28 +434,28 @@ namespace xt
     /**
      * The extended assignment operator.
      */
-    template <class C>
+    template <class C, class EA, class SA>
     template <class E>
-    inline auto xarray_adaptor<C>::operator=(const xexpression<E>& e) -> self_type&
+    inline auto xarray_adaptor<C, EA, SA>::operator=(const xexpression<E>& e) -> self_type&
     {
         return semantic_base::operator=(e);
     }
     //@}
 
-    template <class C>
-    inline auto xarray_adaptor<C>::data_impl() -> container_type&
+    template <class C, class EA, class SA>
+    inline auto xarray_adaptor<C, EA, SA>::data_impl() noexcept -> container_type&
     {
         return m_data;
     }
 
-    template <class C>
-    inline auto xarray_adaptor<C>::data_impl() const -> const container_type&
+    template <class C, class EA, class SA>
+    inline auto xarray_adaptor<C, EA, SA>::data_impl() const noexcept -> const container_type&
     {
         return m_data;
     }
 
-    template <class C>
-    inline void xarray_adaptor<C>::assign_temporary_impl(temporary_type& tmp)
+    template <class C, class EA, class SA>
+    inline void xarray_adaptor<C, EA, SA>::assign_temporary_impl(temporary_type& tmp)
     {
         // TODO (performance improvement) : consider moving tmps
         // shape and strides
@@ -461,9 +463,8 @@ namespace xt
         base_type::get_strides() = tmp.strides();
         base_type::get_backstrides() = tmp.backstrides();
         m_data.resize(tmp.size());
-        std::copy(tmp.data().begin(), tmp.data().end(), m_data.begin());
+        std::copy(tmp.data().cbegin(), tmp.data().cend(), m_data.begin());
     }
 }
 
 #endif
-
