@@ -9,7 +9,8 @@
 #ifndef XASSIGN_HPP
 #define XASSIGN_HPP
 
-#include "xindex.hpp"
+#include <algorithm>
+
 #include "xiterator.hpp"
 
 namespace xt
@@ -52,6 +53,7 @@ namespace xt
         using lhs_iterator = typename E1::stepper;
         using rhs_iterator = typename E2::const_stepper;
         using shape_type = typename E1::shape_type;
+        using index_type = xindex_type_t<shape_type>;
         using size_type = typename lhs_iterator::size_type;
 
         data_assigner(E1& e1, const E2 & e2);
@@ -71,7 +73,7 @@ namespace xt
         rhs_iterator m_rhs;
         rhs_iterator m_rhs_end;
 
-        shape_type m_index;
+        index_type m_index;
     };
 
     /***********************************
@@ -86,11 +88,10 @@ namespace xt
         bool trivial_broadcast = trivial && de2.is_trivial_broadcast(de1.strides());
         if(trivial_broadcast)
         {
-            std::copy(de2.storage_begin(), de2.storage_end(), de1.storage_begin());
+            std::copy(de2.storage_cbegin(), de2.storage_cend(), de1.storage_begin());
         }
         else
         {
-            const auto& shape = de1.shape();
             data_assigner<E1, E2> assigner(de1, de2);
             assigner.run();
         }
@@ -103,7 +104,7 @@ namespace xt
         using size_type = typename E1::size_type;
         const E2& de2 = e2.derived_cast();
         size_type size = de2.dimension();
-        shape_type shape(size, size_type(1));
+        shape_type shape = make_sequence<shape_type>(size, size_type(1));
         bool trivial_broadcast = de2.broadcast_shape(shape);
         e1.derived_cast().reshape(shape);
         return trivial_broadcast;
@@ -145,7 +146,7 @@ namespace xt
     inline void scalar_computed_assign(xexpression<E1>& e1, const E2& e2, F&& f)
     {
         E1& d = e1.derived_cast();
-        std::transform(d.storage_begin(), d.storage_end(), d.storage_begin(),
+        std::transform(d.storage_cbegin(), d.storage_cend(), d.storage_begin(),
                 [e2, &f](const auto& v) { return f(v, e2); });
     }
 
@@ -161,7 +162,7 @@ namespace xt
         de2.broadcast_shape(shape);
         if(shape.size() > de1.shape().size() || shape > de1.shape())
         {
-            throw broadcast_error<size_type>(shape, de1.shape());
+            throw broadcast_error(shape, de1.shape());
         }
     }
 
@@ -173,7 +174,7 @@ namespace xt
     inline data_assigner<E1, E2>::data_assigner(E1& e1, const E2& e2)
         : m_e1(e1), m_lhs(e1.stepper_begin(e1.shape())),
           m_rhs(e2.stepper_begin(e1.shape())), m_rhs_end(e2.stepper_end(e1.shape())),
-          m_index(e1.shape().size(), size_type(0))
+          m_index(make_sequence<index_type>(e1.shape().size(), size_type(0)))
     {
     }
 
