@@ -194,12 +194,6 @@ namespace xt
         strides_type m_backstrides;
     };
 
-    template <class D1, class D2>
-    bool operator==(const xcontainer<D1>& lhs, const xcontainer<D2>& rhs);
-
-    template <class D1, class D2>
-    bool operator!=(const xcontainer<D1>& lhs, const xcontainer<D2>& rhs);
-
     /******************************
      * xcontainer implementation *
      ******************************/
@@ -397,7 +391,7 @@ namespace xt
     }
 
     /**
-     * Transposes the container inplace by permuting the shape with @permutation.
+     * Transposes the container inplace by permuting the shape with @p permutation.
      * @param permutation the vector containing permutation
      * @param check_policy the check level (check_policy::full() or check_policy::none())
      * @tparam Tag selects the level of error checking on permutation vector defaults to check_policy::none.
@@ -520,8 +514,8 @@ namespace xt
     /**
      * Returns a reference to the element at the specified position in the container.
      * @param first iterator starting the sequence of indices
-     * @param second iterator starting the sequence of indices
-     * The number of indices in the squence should be equal or greater
+     * @param last iterator ending the sequence of indices
+     * The number of indices in the squence should be equal to or greater
      * than the number of dimensions of the container.
      */
     template <class D>
@@ -534,8 +528,8 @@ namespace xt
     /**
      * Returns a reference to the element at the specified position in the container.
      * @param first iterator starting the sequence of indices
-     * @param second iterator starting the sequence of indices
-     * The number of indices in the squence should be equal or greater
+     * @param last iterator ending the sequence of indices
+     * The number of indices in the squence should be equal to or greater
      * than the number of dimensions of the container.
      */
     template <class D>
@@ -838,36 +832,6 @@ namespace xt
     }
     //@}
 
-    /**************
-     * comparison *
-     **************/
-
-    /**
-     * @memberof xcontainer
-     * Compares the content of two containers.
-     * @param lhs the first container
-     * @param rhs the second container
-     * @return true if the container are equals
-     */
-    template <class D1, class D2>
-    inline bool operator==(const xcontainer<D1>& lhs, const xcontainer<D2>& rhs)
-    {
-        return lhs.shape() == rhs.shape() && lhs.strides() == rhs.strides()
-            && lhs.data() == rhs.data();
-    }
-
-    /**
-     * @memberof xcontainer
-     * Compares the content of two containers.
-     * @param lhs the first container
-     * @param rhs the second container
-     * @return true if the container are different
-     */
-    template <class D1, class D2>
-    inline bool operator!=(const xcontainer<D1>& lhs, const xcontainer<D2>& rhs)
-    {
-        return !(lhs == rhs);
-    }
 }
 
 #endif
