@@ -65,7 +65,7 @@ namespace xt
     }
 
     template <class C>
-    using get_index_type = typename detail::index_type_impl<C>::type;
+    using xindex_type_t = typename detail::index_type_impl<C>::type;
  
     template <class C>
     class xstepper
@@ -83,6 +83,7 @@ namespace xt
         using size_type = typename container_type::size_type;
         using shape_type = typename container_type::shape_type;
 
+        xstepper() = default;
         xstepper(container_type* c, subiterator_type it, size_type offset) noexcept;
 
         reference operator*() const;
@@ -136,8 +137,9 @@ namespace xt
         using iterator_category = std::forward_iterator_tag;
 
         using shape_type = S;
-        using index_type = get_index_type<shape_type>;
+        using index_type = xindex_type_t<shape_type>;
 
+        xiterator() = default;
         xiterator(It it, const shape_type& shape);
 
         self_type& operator++();
